local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikActionbar = BaikCore.BaikActionbar
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Option = BaikCore.Option
local Ui = BaikCore.Ui

-- Add Module
local Border = BaikActionbar:NewModule("Border")
BaikActionbar.Border = Border

Border.DEFAULT = false

-- Private Variables
local borderFunc = {}

local OPTION_NAME = "border_style"
local SUFFIX_DB_BORDER = "%s_border"
local DB_PREFIX = "action_global"

-- Private Methods
local function GetDbBorder(dbPrefix)
    return string.format(SUFFIX_DB_BORDER, dbPrefix)
end

local function GetGlobalBorder()
    local globalBorder = GetDbBorder(DB_PREFIX)

    return Database:Get(globalBorder)
end

local function GetBorder(dbPrefix)
    local dbBorder = GetDbBorder(dbPrefix)

    return GetGlobalBorder() or Database:Get(dbBorder)
end

local function SetupLocked()
    for _, func in pairs(borderFunc) do
        func()
    end
end

local function Setup()
    if CombatSafe:InLockdown() then
        return
    end

    SetupLocked()
end

local function GetOpts()
    local dbBorder = GetDbBorder(DB_PREFIX)
    return {
        name = "Border",
        type = "group",
        args = {
            hide = {
                name = "Remove",
                type = "toggle",
                get = function(...)
                    return Database:Get(dbBorder)
                end,
                set = function(self, value, ...)
                    Database:Set(dbBorder, value)
                    Setup()
                end,
            }
        }
    }
end

-- Border API
local function HideLocked(frame, dbPrefix, force)
    if GetBorder(dbPrefix) or force then
        Ui:HideFinal(frame)
    else
        Ui:UndoHideFinal(frame)
    end
end

function Border:RemoveButton(btn, dbPrefix, force)
    Assert:Table(btn)
    Assert:String(dbPrefix)

    if CombatSafe:InLockdown() then
        return
    end

    HideLocked(btn.NormalTexture, dbPrefix, force)
end

function Border:RemoveButtonAll(btnTable, dbPrefix, force)
    Assert:Table(btnTable)
    Assert:String(dbPrefix)

    if CombatSafe:InLockdown() then
        return
    end

    for _, btn in pairs(btnTable) do
        HideLocked(btn.NormalTexture, dbPrefix, force)
    end
end

function Border:RemoveFrame(frame, dbPrefix, force)
    Assert:Table(frame)
    Assert:String(dbPrefix)

    if CombatSafe:InLockdown() then
        return
    end

    HideLocked(frame, dbPrefix, force)
end

function Border:RemoveFrames(frameTbl, dbPrefix, force)
    Assert:Table(frameTbl)
    Assert:String(dbPrefix)

    if CombatSafe:InLockdown() then
        return
    end

    for _, frame in pairs(frameTbl) do
        HideLocked(frame, dbPrefix, force)
    end
end

function Border:RemoveAlpha(frame, dbPrefix, force)
    Assert:Table(frame)
    Assert:String(dbPrefix)

    local alpha = (GetBorder(dbPrefix) or force) and 0 or 1
    frame:SetAlpha(alpha)
end

function Border:RemoveAlphas(frameTbl, dbPrefix, force)
    Assert:Table(frameTbl)
    Assert:String(dbPrefix)

    for _, frame in pairs(frameTbl) do
        self:RemoveAlpha(frame, dbPrefix, force)
    end
end

function Border:IsHide(dbPrefix)
    return GetBorder(dbPrefix)
end

function Border:InitDb(dbPrefix)
    Assert:String(dbPrefix)
    local dbBorder = GetDbBorder(dbPrefix)

    Database:InitIfEmpty(dbBorder, self.DEFAULT)
end

function Border:GetOpts(dbPrefix, SetBorderFunc)
    borderFunc[dbPrefix] = SetBorderFunc
    local dbBorder = GetDbBorder(dbPrefix)

    return {
        name = "Remove Border",
        type = "toggle",
        disabled = GetGlobalBorder,
        get = function(...)
            return Database:Get(dbBorder)
        end,
        set = function(self, value, ...)
            Database:Set(dbBorder, value)
            if not CombatSafe:InLockdown() then
                SetBorderFunc()
            end
        end
    }
end

-- Ace3 Callbacks
function Border:OnInitialize()
    Log:v("Border_OnInitialize")
    BaikActionbar:AddOption(OPTION_NAME, GetOpts())
    CombatSafe:Setup(SetupLocked)
end

function Border:OnEnable()
    Log:v("Border_OnEnable")
end

function Border:OnDisable()
    Log:v("Border_OnDisable")
end
